@@ -48,7 +48,7 @@ mktime_from_string (char *time_s,
 		    char *month_s,
 		    char *year_s)
 {
-	static char  *months[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", 
+	static char  *months[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
 				   "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
 	struct tm    tm = {0, };
 	char       **fields;
@@ -91,9 +91,9 @@ ar_get_last_field (const char *line,
 		   int         field_n)
 {
 	const char *f_start, *f_end;
-	
+
 	line = line + start_from;
-        
+
 	f_start = line;
 	while ((*f_start == ' ') && (*f_start != *line))
 		f_start++;
@@ -110,13 +110,13 @@ ar_get_last_field (const char *line,
 		} else
 			f_end++;
 	}
-	
+
 	return g_strdup (f_start);
 }
 
 
 static void
-process_line (char     *line, 
+process_line (char     *line,
 	      gpointer  data)
 {
 	FileData    *fdata;
@@ -182,22 +182,21 @@ process_line (char     *line,
 
 	if (*fdata->name == 0)
 		file_data_free (fdata);
-	else 
+	else
 		fr_command_add_file (comm, fdata);
 }
 
 
 static void
-fr_command_ar_list (FrCommand  *comm,
-		    const char *password)
+fr_command_ar_list (FrCommand  *comm)
 {
-	fr_process_set_out_line_func (FR_COMMAND (comm)->process, 
+	fr_process_set_out_line_func (FR_COMMAND (comm)->process,
 				      process_line,
 				      comm);
 
 	fr_process_begin_command (comm->process, "ar");
 	fr_process_add_arg (comm->process, "tv");
-	fr_process_add_arg (comm->process, comm->e_filename);
+	fr_process_add_arg (comm->process, comm->filename);
 	fr_process_end_command (comm->process);
 	fr_process_start (comm->process);
 }
@@ -205,11 +204,11 @@ fr_command_ar_list (FrCommand  *comm,
 
 static void
 fr_command_ar_add (FrCommand     *comm,
+		   const char    *from_file,
 		   GList         *file_list,
 		   const char    *base_dir,
 		   gboolean       update,
-		   const char    *password,
-		   FrCompression  compression)
+		   gboolean       recursive)
 {
 	GList *scan;
 
@@ -220,29 +219,28 @@ fr_command_ar_add (FrCommand     *comm,
 	else
 		fr_process_add_arg (comm->process, "r");
 
-	if (base_dir != NULL) 
+	if (base_dir != NULL)
 		fr_process_set_working_dir (comm->process, base_dir);
 
-	fr_process_add_arg (comm->process, comm->e_filename);
+	fr_process_add_arg (comm->process, comm->filename);
 
-	for (scan = file_list; scan; scan = scan->next) 
-		fr_process_add_arg (comm->process, (gchar*) scan->data);
+	for (scan = file_list; scan; scan = scan->next)
+		fr_process_add_arg (comm->process, scan->data);
 
 	fr_process_end_command (comm->process);
 }
 
 
 static void
-fr_command_ar_delete (FrCommand *comm,
-		      GList     *file_list)
+fr_command_ar_delete (FrCommand  *comm,
+		      const char *from_file,
+		      GList      *file_list)
 {
 	GList *scan;
 
 	fr_process_begin_command (comm->process, "ar");
 	fr_process_add_arg (comm->process, "d");
-
-	fr_process_add_arg (comm->process, comm->e_filename);
-
+	fr_process_add_arg (comm->process, comm->filename);
 	for (scan = file_list; scan; scan = scan->next)
 		fr_process_add_arg (comm->process, scan->data);
 	fr_process_end_command (comm->process);
@@ -251,51 +249,61 @@ fr_command_ar_delete (FrCommand *comm,
 
 static void
 fr_command_ar_extract (FrCommand  *comm,
+		       const char *from_file,
 		       GList      *file_list,
 		       const char *dest_dir,
 		       gboolean    overwrite,
 		       gboolean    skip_older,
-		       gboolean    junk_paths,
-		       const char *password)
+		       gboolean    junk_paths)
 {
 	GList *scan;
 
 	fr_process_begin_command (comm->process, "ar");
 
-	if (dest_dir != NULL) 
+	if (dest_dir != NULL)
 		fr_process_set_working_dir (comm->process, dest_dir);
 
 	fr_process_add_arg (comm->process, "x");
-	fr_process_add_arg (comm->process, comm->e_filename);
-
+	fr_process_add_arg (comm->process, comm->filename);
 	for (scan = file_list; scan; scan = scan->next)
 		fr_process_add_arg (comm->process, scan->data);
-
 	fr_process_end_command (comm->process);
 }
 
 
 static void
-fr_command_ar_handle_error (FrCommand   *comm, 
+fr_command_ar_handle_error (FrCommand   *comm,
 			    FrProcError *error)
 {
 	/* FIXME */
 }
 
 
-static void
-fr_command_ar_set_mime_type (FrCommand  *comm,
-			     const char *mime_type)
+const char *ar_mime_type[] = { "application/x-ar", NULL };
+
+
+const char **
+fr_command_ar_get_mime_types (FrCommand *comm)
 {
-	FR_COMMAND_CLASS (parent_class)->set_mime_type (comm, mime_type);
-	
-	comm->capabilities |= FR_COMMAND_CAP_ARCHIVE_MANY_FILES;
-	if (is_program_in_path ("ar")) 
-		comm->capabilities |= FR_COMMAND_CAP_READ_WRITE;
+	return ar_mime_type;
 }
 
 
-static void 
+FrCommandCap
+fr_command_ar_get_capabilities (FrCommand  *comm,
+			        const char *mime_type)
+{
+	FrCommandCap capabilities;
+
+	capabilities = FR_COMMAND_CAN_ARCHIVE_MANY_FILES;
+	if (is_program_in_path ("ar"))
+		capabilities |= FR_COMMAND_CAN_READ_WRITE;
+
+	return capabilities;
+}
+
+
+static void
 fr_command_ar_class_init (FrCommandArClass *class)
 {
         GObjectClass   *gobject_class = G_OBJECT_CLASS (class);
@@ -306,19 +314,19 @@ fr_command_ar_class_init (FrCommandArClass *class)
 
 	gobject_class->finalize = fr_command_ar_finalize;
 
-        afc->list           = fr_command_ar_list;
-	afc->add            = fr_command_ar_add;
-	afc->delete         = fr_command_ar_delete;
-	afc->extract        = fr_command_ar_extract;
-	afc->handle_error   = fr_command_ar_handle_error;
-	afc->set_mime_type  = fr_command_ar_set_mime_type;
+        afc->list             = fr_command_ar_list;
+	afc->add              = fr_command_ar_add;
+	afc->delete           = fr_command_ar_delete;
+	afc->extract          = fr_command_ar_extract;
+	afc->handle_error     = fr_command_ar_handle_error;
+	afc->get_mime_types   = fr_command_ar_get_mime_types;
+	afc->get_capabilities = fr_command_ar_get_capabilities;
 }
 
- 
-static void 
+
+static void
 fr_command_ar_init (FrCommand *comm)
 {
-	comm->propCanModify                = TRUE;
 	comm->propAddCanUpdate             = TRUE;
 	comm->propAddCanReplace            = TRUE;
 	comm->propAddCanStoreFolders       = FALSE;
@@ -330,12 +338,12 @@ fr_command_ar_init (FrCommand *comm)
 }
 
 
-static void 
+static void
 fr_command_ar_finalize (GObject *object)
 {
         g_return_if_fail (object != NULL);
         g_return_if_fail (FR_IS_COMMAND_AR (object));
-	
+
 	/* Chain up */
         if (G_OBJECT_CLASS (parent_class)->finalize)
 		G_OBJECT_CLASS (parent_class)->finalize (object);
