@@ -20,17 +20,20 @@
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  */
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 
 #include <glib.h>
+#include <glib/gi18n.h>
 
 #include "file-data.h"
 #include "file-utils.h"
 #include "glib-utils.h"
 #include "fr-command.h"
 #include "fr-command-rar.h"
+#include "fr-error.h"
 
 static void fr_command_rar_class_init  (FrCommandRarClass *class);
 static void fr_command_rar_init        (FrCommand         *afile);
@@ -103,6 +106,8 @@ process_line (char     *line,
 			rar_comm->list_started = TRUE;
 			rar_comm->odd_line = TRUE;
 		}
+		else if (strncmp (line, "Volume ", 7) == 0)
+			rar_comm->multi_volume = TRUE;
 		return;
 	}
 
@@ -125,7 +130,7 @@ process_line (char     *line,
 		if (*name_field == '/') {
 			fdata->full_path = g_strdup (name_field);
 			fdata->original_path = fdata->full_path;
-		} 
+		}
 		else {
 			fdata->full_path = g_strconcat ("/", name_field, NULL);
 			fdata->original_path = fdata->full_path + 1;
@@ -133,7 +138,7 @@ process_line (char     *line,
 
 		fdata->link = NULL;
 		fdata->path = remove_level_from_path (fdata->full_path);
-	} 
+	}
 	else {
 		FileData *fdata;
 
@@ -143,28 +148,38 @@ process_line (char     *line,
 
 		fields = split_line (line, 6);
 
-		fdata->size = g_ascii_strtoull (fields[0], NULL, 10);
-		fdata->modified = mktime_from_string (fields[3], fields[4]);
+		if ((strcmp (fields[2], "<->") == 0)
+		    || (strcmp (fields[2], "<--") == 0))
+		{
+			/* ignore files that span more volumes */
 
-		if ((fields[5][1] == 'D') || (fields[5][0] == 'd')) {
-			char *tmp;
-			
-			tmp = fdata->full_path;
-			fdata->full_path = g_strconcat (fdata->full_path, "/", NULL);
-			
-			fdata->original_path = g_strdup (fdata->original_path);
-			fdata->free_original_path = TRUE;
-			
-			g_free (tmp);
-			
-			fdata->name = dir_name_from_path (fdata->full_path);
-			fdata->dir = TRUE;
-		} 
-		else
-			fdata->name = g_strdup (file_name_from_path (fdata->full_path));
+			file_data_free (rar_comm->fdata);
+			rar_comm->fdata = NULL;
+		}
+		else {
+			fdata->size = g_ascii_strtoull (fields[0], NULL, 10);
+			fdata->modified = mktime_from_string (fields[3], fields[4]);
 
-		fr_command_add_file (comm, fdata);
-		rar_comm->fdata = NULL;
+			if ((fields[5][1] == 'D') || (fields[5][0] == 'd')) {
+				char *tmp;
+
+				tmp = fdata->full_path;
+				fdata->full_path = g_strconcat (fdata->full_path, "/", NULL);
+
+				fdata->original_path = g_strdup (fdata->original_path);
+				fdata->free_original_path = TRUE;
+
+				g_free (tmp);
+
+				fdata->name = dir_name_from_path (fdata->full_path);
+				fdata->dir = TRUE;
+			}
+			else
+				fdata->name = g_strdup (file_name_from_path (fdata->full_path));
+
+			fr_command_add_file (comm, fdata);
+			rar_comm->fdata = NULL;
+		}
 
 		g_strfreev (fields);
 	}
@@ -178,18 +193,22 @@ add_password_arg (FrCommand  *comm,
 		  const char *password,
 		  gboolean    disable_query)
 {
-	if ((password != NULL) && (password[0] != '\0'))
-		fr_process_add_arg_concat (comm->process, "-p", password, NULL); 
+	if ((password != NULL) && (password[0] != '\0')) {
+		if (comm->encrypt_header)
+			fr_process_add_arg_concat (comm->process, "-hp", password, NULL);
+		else
+			fr_process_add_arg_concat (comm->process, "-p", password, NULL);
+	}
 	else if (disable_query)
 		fr_process_add_arg (comm->process, "-p-");
 }
 
 
 static void
-fr_command_rar_list (FrCommand  *comm,
-		     const char *password)
+fr_command_rar_list (FrCommand  *comm)
 {
 	FR_COMMAND_RAR (comm)->list_started = FALSE;
+	FR_COMMAND_RAR (comm)->multi_volume = FALSE;
 
 	fr_process_set_out_line_func (FR_COMMAND (comm)->process,
 				      process_line,
@@ -201,8 +220,9 @@ fr_command_rar_list (FrCommand  *comm,
 		fr_process_begin_command (comm->process, "unrar");
 	fr_process_add_arg (comm->process, "v");
 	fr_process_add_arg (comm->process, "-c-");
+	fr_process_add_arg (comm->process, "-v");
 
-	add_password_arg (comm, password, TRUE);
+	add_password_arg (comm, comm->password, TRUE);
 
 	/* stop switches scanning */
 	fr_process_add_arg (comm->process, "--");
@@ -213,17 +233,72 @@ fr_command_rar_list (FrCommand  *comm,
 }
 
 
+static char Progress_Message[4196];
+static char Progress_Filename[4096];
+
+
+static void
+parse_progress_line (FrCommand  *comm,
+		     const char *prefix,
+		     const char *message_prefix,
+		     const char *line)
+{
+	int prefix_len;
+
+	prefix_len = strlen (prefix);
+	if (strncmp (line, prefix, prefix_len) == 0) {
+		double fraction;
+		int    len;
+
+		strcpy (Progress_Filename, line + prefix_len);
+		len = strlen (Progress_Filename);
+		if ((len > 5) && (strncmp (Progress_Filename + len - 5, "  OK ", 5) == 0))
+			Progress_Filename[len - 5] = 0;
+		sprintf (Progress_Message, "%s%s", message_prefix, file_name_from_path (Progress_Filename));
+		fr_command_message (comm, Progress_Message);
+
+		fraction= (double) ++comm->n_file / (comm->n_files + 1);
+		fr_command_progress (comm, fraction);
+	}
+}
+
+
+static void
+process_line__add (char     *line,
+		   gpointer  data)
+{
+	FrCommand *comm = FR_COMMAND (data);
+
+	if (strncmp (line, "Creating archive ", 17) == 0) {
+		char *uri;
+
+		uri = g_filename_to_uri (line + 17, NULL, NULL);
+		fr_command_working_archive (comm, uri);
+		g_free (uri);
+
+		return;
+	}
+
+	if (comm->n_files != 0)
+		parse_progress_line (comm, "Adding    ", _("Adding file: "), line);
+}
+
+
 static void
 fr_command_rar_add (FrCommand     *comm,
+		    const char    *from_file,
 		    GList         *file_list,
 		    const char    *base_dir,
 		    gboolean       update,
-		    gboolean       recursive,
-		    const char    *password,
-		    FrCompression  compression)
+		    gboolean       recursive)
 {
 	GList *scan;
 
+	fr_process_use_standard_locale (comm->process, TRUE);
+	fr_process_set_out_line_func (comm->process,
+				      process_line__add,
+				      comm);
+
 	fr_process_begin_command (comm->process, "rar");
 
 	if (base_dir != NULL)
@@ -234,7 +309,7 @@ fr_command_rar_add (FrCommand     *comm,
 	else
 		fr_process_add_arg (comm->process, "a");
 
-	switch (compression) {
+	switch (comm->compression) {
 	case FR_COMPRESSION_VERY_FAST:
 		fr_process_add_arg (comm->process, "-m1"); break;
 	case FR_COMPRESSION_FAST:
@@ -245,54 +320,113 @@ fr_command_rar_add (FrCommand     *comm,
 		fr_process_add_arg (comm->process, "-m5"); break;
 	}
 
-	add_password_arg (comm, password, FALSE);
+	add_password_arg (comm, comm->password, FALSE);
+
+	if (comm->volume_size > 0)
+		fr_process_add_arg_printf (comm->process, "-v%ub", comm->volume_size);
 
 	/* disable percentage indicator */
 	fr_process_add_arg (comm->process, "-Idp");
 
-	/* stop switches scanning */
 	fr_process_add_arg (comm->process, "--");
-
 	fr_process_add_arg (comm->process, comm->filename);
 
-	for (scan = file_list; scan; scan = scan->next)
-		fr_process_add_arg (comm->process, scan->data);
+	if (from_file == NULL)
+		for (scan = file_list; scan; scan = scan->next)
+			fr_process_add_arg (comm->process, scan->data);
+	else
+		fr_process_add_arg_concat (comm->process, "@", from_file, NULL);
 
 	fr_process_end_command (comm->process);
 }
 
 
 static void
-fr_command_rar_delete (FrCommand *comm,
-		       GList     *file_list)
+process_line__delete (char     *line,
+		      gpointer  data)
+{
+	FrCommand *comm = FR_COMMAND (data);
+
+	if (strncmp (line, "Deleting from ", 14) == 0) {
+		char *uri;
+
+		uri = g_filename_to_uri (line + 14, NULL, NULL);
+		fr_command_working_archive (comm, uri);
+		g_free (uri);
+
+		return;
+	}
+
+	if (comm->n_files != 0)
+		parse_progress_line (comm, "Deleting ", _("Removing file: "), line);
+}
+
+
+static void
+fr_command_rar_delete (FrCommand  *comm,
+		       const char *from_file,
+		       GList      *file_list)
 {
 	GList *scan;
 
+	fr_process_use_standard_locale (comm->process, TRUE);
+	fr_process_set_out_line_func (comm->process,
+				      process_line__delete,
+				      comm);
+
 	fr_process_begin_command (comm->process, "rar");
 	fr_process_add_arg (comm->process, "d");
 
-	/* stop switches scanning */
 	fr_process_add_arg (comm->process, "--");
-
 	fr_process_add_arg (comm->process, comm->filename);
 
-	for (scan = file_list; scan; scan = scan->next)
-		fr_process_add_arg (comm->process, scan->data);
+	if (from_file == NULL)
+		for (scan = file_list; scan; scan = scan->next)
+			fr_process_add_arg (comm->process, scan->data);
+	else
+		fr_process_add_arg_concat (comm->process, "@", from_file, NULL);
+
 	fr_process_end_command (comm->process);
 }
 
 
+static void
+process_line__extract (char     *line,
+		       gpointer  data)
+{
+	FrCommand *comm = FR_COMMAND (data);
+
+	if (strncmp (line, "Extracting from ", 16) == 0) {
+		char *uri;
+
+		uri = g_filename_to_uri (line + 16, NULL, NULL);
+		fr_command_working_archive (comm, uri);
+		g_free (uri);
+
+		return;
+	}
+
+	if (comm->n_files != 0)
+		parse_progress_line (comm, "Extracting  ", _("Extracting file: "), line);
+}
+
+
 static void
 fr_command_rar_extract (FrCommand  *comm,
+			const char *from_file,
 			GList      *file_list,
 			const char *dest_dir,
 			gboolean    overwrite,
 			gboolean    skip_older,
-			gboolean    junk_paths,
-			const char *password)
+			gboolean    junk_paths)
 {
 	GList *scan;
 
+	fr_process_use_standard_locale (comm->process, TRUE);
+	fr_process_set_out_line_func (comm->process,
+				      process_line__extract,
+				      comm);
+
 	if (have_rar ())
 		fr_process_begin_command (comm->process, "rar");
 	else
@@ -311,18 +445,19 @@ fr_command_rar_extract (FrCommand  *comm,
 	if (junk_paths)
 		fr_process_add_arg (comm->process, "-ep");
 
-	add_password_arg (comm, password, TRUE);
+	add_password_arg (comm, comm->password, TRUE);
 
 	/* disable percentage indicator */
 	fr_process_add_arg (comm->process, "-Idp");
 
-	/* stop switches scanning */
 	fr_process_add_arg (comm->process, "--");
-
 	fr_process_add_arg (comm->process, comm->filename);
 
-	for (scan = file_list; scan; scan = scan->next)
-		fr_process_add_arg (comm->process, scan->data);
+	if (from_file == NULL)
+		for (scan = file_list; scan; scan = scan->next)
+			fr_process_add_arg (comm->process, scan->data);
+	else
+		fr_process_add_arg_concat (comm->process, "@", from_file, NULL);
 
 	if (dest_dir != NULL)
 		fr_process_add_arg (comm->process, dest_dir);
@@ -332,8 +467,7 @@ fr_command_rar_extract (FrCommand  *comm,
 
 
 static void
-fr_command_rar_test (FrCommand   *comm,
-		     const char  *password)
+fr_command_rar_test (FrCommand   *comm)
 {
 	if (have_rar ())
 		fr_process_begin_command (comm->process, "rar");
@@ -342,7 +476,7 @@ fr_command_rar_test (FrCommand   *comm,
 
 	fr_process_add_arg (comm->process, "t");
 
-	add_password_arg (comm, password, TRUE);
+	add_password_arg (comm, comm->password, TRUE);
 
 	/* disable percentage indicator */
 	fr_process_add_arg (comm->process, "-Idp");
@@ -359,41 +493,78 @@ static void
 fr_command_rar_handle_error (FrCommand   *comm,
 			     FrProcError *error)
 {
-	if (error->type != FR_PROC_ERROR_COMMAND_ERROR) 
-		return;
-		
-	if (error->status == 3) {
-		error->type = FR_PROC_ERROR_ASK_PASSWORD;
-	}
-	else {
+	GList *scan;
+
+#if 0
+	{
 		GList *scan;
-		
-		if (error->status <= 1)
-			error->type = FR_PROC_ERROR_NONE;
-			
-		for (scan = g_list_last (comm->process->err.raw); scan; scan = scan->prev) {
-			char *line = scan->data;
-				
-			if (strstr (line, "password incorrect") != NULL) {
-				error->type = FR_PROC_ERROR_ASK_PASSWORD;
-				break;
-			}
+
+		for (scan = g_list_last (comm->process->err.raw); scan; scan = scan->prev)
+			g_print ("%s\n", (char*)scan->data);
+	}
+#endif
+
+	if (error->type != FR_PROC_ERROR_COMMAND_ERROR)
+		return;
+
+	if (error->status == 3)
+		error->type = FR_PROC_ERROR_ASK_PASSWORD;
+	else if (error->status <= 1)
+		error->type = FR_PROC_ERROR_NONE;
+
+	for (scan = g_list_last (comm->process->err.raw); scan; scan = scan->prev) {
+		char *line = scan->data;
+
+		if (strstr (line, "password incorrect") != NULL) {
+			error->type = FR_PROC_ERROR_ASK_PASSWORD;
+			break;
+		}
+
+		if (strncmp (line, "Cannot find volume", 18) == 0) {
+			char *volume_filename;
+
+			g_clear_error (&error->gerror);
+
+			error->type = FR_PROC_ERROR_MISSING_VOLUME;
+			volume_filename = g_path_get_basename (line + strlen ("Cannot find volume "));
+			error->gerror = g_error_new (FR_ERROR, error->status, _("Could not find the volume: %s"), volume_filename);
+			g_free (volume_filename);
+			break;
 		}
 	}
 }
 
 
-static void
-fr_command_rar_set_mime_type (FrCommand  *comm,
-		 	      const char *mime_type)
+const char *rar_mime_type[] = { "application/x-cbr",
+				"application/x-rar",
+				NULL };
+
+
+const char **
+fr_command_rar_get_mime_types (FrCommand *comm)
 {
-	FR_COMMAND_CLASS (parent_class)->set_mime_type (comm, mime_type);
-	
-	comm->capabilities |= FR_COMMAND_CAP_ARCHIVE_MANY_FILES;
-	if (is_program_in_path ("rar")) 
-		comm->capabilities |= FR_COMMAND_CAP_READ_WRITE;
-	else if (is_program_in_path ("unrar")) 
-		comm->capabilities |= FR_COMMAND_CAP_READ;
+	return rar_mime_type;
+}
+
+
+FrCommandCap
+fr_command_rar_get_capabilities (FrCommand  *comm,
+			         const char *mime_type)
+{
+	FrCommandRar *comm_rar = FR_COMMAND_RAR (comm);
+	FrCommandCap  capabilities;
+
+	capabilities = FR_COMMAND_CAN_ARCHIVE_MANY_FILES | FR_COMMAND_CAN_ENCRYPT | FR_COMMAND_CAN_ENCRYPT_HEADER;
+	if (is_program_in_path ("rar"))
+		capabilities |= FR_COMMAND_CAN_READ_WRITE | FR_COMMAND_CAN_CREATE_VOLUMES;
+	else if (is_program_in_path ("unrar"))
+		capabilities |= FR_COMMAND_CAN_READ;
+
+	/* multi-volumes are read-only */
+	if ((comm->files->len > 0) && comm_rar->multi_volume && (capabilities & FR_COMMAND_CAN_WRITE))
+		capabilities ^= FR_COMMAND_CAN_WRITE;
+
+	return capabilities;
 }
 
 
@@ -408,13 +579,14 @@ fr_command_rar_class_init (FrCommandRarClass *class)
 
 	gobject_class->finalize = fr_command_rar_finalize;
 
-	afc->list           = fr_command_rar_list;
-	afc->add            = fr_command_rar_add;
-	afc->delete         = fr_command_rar_delete;
-	afc->extract        = fr_command_rar_extract;
-	afc->test           = fr_command_rar_test;
-	afc->handle_error   = fr_command_rar_handle_error;
-	afc->set_mime_type  = fr_command_rar_set_mime_type;
+	afc->list             = fr_command_rar_list;
+	afc->add              = fr_command_rar_add;
+	afc->delete           = fr_command_rar_delete;
+	afc->extract          = fr_command_rar_extract;
+	afc->test             = fr_command_rar_test;
+	afc->handle_error     = fr_command_rar_handle_error;
+	afc->get_mime_types   = fr_command_rar_get_mime_types;
+	afc->get_capabilities = fr_command_rar_get_capabilities;
 }
 
 
@@ -429,6 +601,7 @@ fr_command_rar_init (FrCommand *comm)
 	comm->propExtractCanJunkPaths      = TRUE;
 	comm->propPassword                 = TRUE;
 	comm->propTest                     = TRUE;
+	comm->propListFromFile             = TRUE;
 }
 
 
