@@ -38,7 +38,7 @@
 typedef struct _FrProcess        FrProcess;
 typedef struct _FrProcessClass   FrProcessClass;
 typedef struct _FrProcessPrivate FrProcessPrivate;
-  
+
 typedef void     (*ProcFunc)     (gpointer data);
 typedef gboolean (*ContinueFunc) (gpointer data);
 typedef void     (*LineFunc)     (char *line, gpointer data);
@@ -55,8 +55,8 @@ struct _FrProcess {
 
 	/*< public >*/
 
-	gboolean          term_on_stop;  /* whether we must terminate the 
-					  * command when calling 
+	gboolean          term_on_stop;  /* whether we must terminate the
+					  * command when calling
 					  * fr_process_stop. */
 
 	/*< public read-only >*/
@@ -67,7 +67,7 @@ struct _FrProcess {
 
 	/*< protected >*/
 
-	gboolean          restart;       /* whether to restart the process 
+	gboolean          restart;       /* whether to restart the process
 			  		  * after an error. */
 
 	FrProcessPrivate *priv;
@@ -86,42 +86,45 @@ struct _FrProcessClass {
 GType       fr_process_get_type             (void);
 FrProcess * fr_process_new                  (void);
 void        fr_process_clear                (FrProcess    *fr_proc);
-void        fr_process_begin_command        (FrProcess    *fr_proc, 
+void        fr_process_begin_command        (FrProcess    *fr_proc,
 					     const char   *arg);
-void        fr_process_begin_command_at     (FrProcess    *fr_proc, 
+void        fr_process_begin_command_at     (FrProcess    *fr_proc,
 					     const char   *arg,
 					     int           index);
-void        fr_process_add_arg              (FrProcess    *fr_proc, 
+void        fr_process_add_arg              (FrProcess    *fr_proc,
 					     const char   *arg);
-void        fr_process_add_arg_concat       (FrProcess    *fr_proc, 
+void        fr_process_add_arg_concat       (FrProcess    *fr_proc,
 					     const char   *arg,
 					     ...);
-void        fr_process_set_arg_at           (FrProcess    *fr_proc, 
+void        fr_process_add_arg_printf       (FrProcess    *fr_proc,
+					     const char   *format,
+					     ...) G_GNUC_PRINTF (2, 3);
+void        fr_process_set_arg_at           (FrProcess    *fr_proc,
 					     int           n_comm,
 					     int           n_arg,
 					     const char   *arg);
-void        fr_process_set_begin_func       (FrProcess    *fr_proc, 
+void        fr_process_set_begin_func       (FrProcess    *fr_proc,
 					     ProcFunc      func,
 					     gpointer      func_data);
-void        fr_process_set_end_func         (FrProcess    *fr_proc, 
+void        fr_process_set_end_func         (FrProcess    *fr_proc,
 					     ProcFunc      func,
 					     gpointer      func_data);
-void        fr_process_set_continue_func    (FrProcess    *fr_proc, 
+void        fr_process_set_continue_func    (FrProcess    *fr_proc,
 					     ContinueFunc  func,
 					     gpointer      func_data);
 void        fr_process_end_command          (FrProcess    *fr_proc);
-void        fr_process_set_working_dir      (FrProcess    *fr_proc, 
+void        fr_process_set_working_dir      (FrProcess    *fr_proc,
 					     const char   *arg);
-void        fr_process_set_sticky           (FrProcess    *fr_proc, 
+void        fr_process_set_sticky           (FrProcess    *fr_proc,
 					     gboolean      sticky);
-void        fr_process_set_ignore_error     (FrProcess    *fr_proc, 
+void        fr_process_set_ignore_error     (FrProcess    *fr_proc,
 					     gboolean      ignore_error);
 void        fr_process_use_standard_locale  (FrProcess    *fr_proc,
 					     gboolean      use_stand_locale);
-void        fr_process_set_out_line_func    (FrProcess    *fr_proc, 
+void        fr_process_set_out_line_func    (FrProcess    *fr_proc,
 					     LineFunc  func,
 					     gpointer      func_data);
-void        fr_process_set_err_line_func    (FrProcess    *fr_proc, 
+void        fr_process_set_err_line_func    (FrProcess    *fr_proc,
 					     LineFunc  func,
 					     gpointer      func_data);
 void        fr_process_start                (FrProcess    *fr_proc);
